﻿using Sandbox.ModAPI;
using System;
using System.Collections.Generic;
using VRage.Game.ModAPI;
using VRage.ModAPI;
using VRage.Game.Components;

namespace Scripts.KSWH
{
    [MySessionComponentDescriptor(MyUpdateOrder.BeforeSimulation)]

    class CargoShips : MySessionComponentBase
    {
        public static String Identifier = "Largeks8b4";

        private static bool WantTalk = false;
        private bool flagOnes = false;

        class MissionData
        {
            public List<EnemyShip> EnemyShips = new List<EnemyShip>();
            public List<IMyCubeGrid> CheckedShips = new List<IMyCubeGrid>();
        }

        public static MyLogger Logger;
        private MissionData m_data;
        private bool m_loaded = false;
        private bool m_loadFailed;
        private int count = 0;
        private HashSet<IMyEntity> m_entitiesCache = new HashSet<IMyEntity>();
        public List<EnemyShip> m_deleteEnemyCache = new List<EnemyShip>();
        
        public override void UpdateBeforeSimulation()
        {
            if (!MyAPIGateway.Multiplayer.IsServer)
            {
                return;
            }

            if (MyAPIGateway.Session == null)
            {
                return;
            }

            if (!m_loaded)
            {
                Init();
            }

            if (!m_loaded)
            {
                return;
            }

            count++;

            UpdateEnemies();
        }

        private void Init()
        {
            if (!MyAPIGateway.Multiplayer.IsServer)
            {
                return;
            }
            Logger = new MyLogger("Log.log");
            m_data = new MissionData();

            m_loaded = true;

            MyAPIGateway.Utilities.MessageEntered += Utilities_MessageEntered;

            InitEnemies();
        }

        private void UpdateEnemies()
        {
            if (!MyAPIGateway.Multiplayer.IsServer)
            {
                return;
            }
            MyAPIGateway.Entities.GetEntities(m_entitiesCache, (x) => x is IMyCubeGrid && (x.DisplayName.Contains(Identifier)) );

            foreach (var ent in m_entitiesCache)
            {
                bool exists = false;
                foreach (var checkedShip in m_data.CheckedShips)
                {
                    if (checkedShip.Equals(ent))
                        exists = true;
                }
                if (!exists)
                {
                    EnemyShip tmpShipEnt = new EnemyShip(ent as IMyCubeGrid, 100000);
                    if (tmpShipEnt.myShip == null
                        //|| tmpShipEnt.myShip.IsTrash()
                        || !tmpShipEnt.myShip.IsVisible()
                        || !tmpShipEnt.IsWorkingInvader())
                        break;

                    MyAPIGateway.Utilities.ShowNotification("New Cargo Ship in Sector", 3000, VRage.Game.MyFontEnum.Red);
                    m_data.EnemyShips.Add(tmpShipEnt);

                    var grid = ent as IMyCubeGrid;

                    m_data.CheckedShips.Add(ent as IMyCubeGrid);
                }
            }
            m_entitiesCache.Clear();

            foreach (var enemyShip in m_data.EnemyShips)
            {
                if (enemyShip.myShip == null
                    //|| enemyShip.myShip.IsTrash()
                    || !enemyShip.myShip.IsVisible()
                    || !enemyShip.IsWorkingInvader())
                    m_deleteEnemyCache.Add(enemyShip);
            }

            foreach (var enemyShip in m_deleteEnemyCache)
                m_data.EnemyShips.Remove(enemyShip);

            m_deleteEnemyCache.Clear();

            foreach (var enemyShip in m_data.EnemyShips)
                enemyShip.Update();
        }

        private void InitEnemies()
        {
            if (!MyAPIGateway.Multiplayer.IsServer)
            {
                return;
            }
            //Find npc player that was created in world in prior
            MyAPIGateway.Entities.GetEntities(m_entitiesCache, (x) => x is IMyCubeGrid && (x.DisplayName.Contains(Identifier)) );

            foreach (var ent in m_entitiesCache)
            {
                m_data.EnemyShips.Add(new EnemyShip(ent as IMyCubeGrid, 100000));

                var grid = ent as IMyCubeGrid;

                m_data.CheckedShips.Add(ent as IMyCubeGrid);
            }
            m_entitiesCache.Clear();
        }

        private void Utilities_MessageEntered(string messageText, ref bool sendToOthers)
        {
            if (!MyAPIGateway.Multiplayer.IsServer)
            {
                return;
            }
            if (!messageText.StartsWith("/setshipname")) { return; }
            List<string> words = new List<string>(messageText.Trim().Replace("/setshipname", "").Split(' '));
            sendToOthers = false;

            Identifier = words[1];
            InitEnemies();
            MyAPIGateway.Utilities.ShowNotification("Ship Name contains \"" + words[1] + "\" want your money!!", 6000, VRage.Game.MyFontEnum.DarkBlue);
        }

        protected override void UnloadData()
        {
            if (!MyAPIGateway.Multiplayer.IsServer)
            {
                return;
            }

            MyAPIGateway.Utilities.MessageEntered -= Utilities_MessageEntered;

            base.UnloadData();
            Logger.Close();
            Logger = null;
        }

        public static void Talk(String who, String what)
        {
            if (!MyAPIGateway.Multiplayer.IsServer)
            {
                return;
            }
            if (!WantTalk)
                return;
            Logger.WriteLine(who + " said: " + what);
            who = who + " said";
            MyAPIGateway.Utilities.ShowMessage(who, what);
        }
    }
}
